@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import with_statement
-import glob
+import exceptions
 import os
 import re
 import shutil
@@ -18,7 +18,7 @@ from log import safe_print
 from meta import name as appname, version
 from options import debug
 from util_decimal import stripzeros
-from util_os import islink, readlink, waccess
+from util_os import islink, readlink, safe_glob, waccess
 from util_str import safe_unicode, strtr
 from worker import (Error, Info, UnloggedInfo, get_current_profile_path,
 					show_result_dialog)
@@ -162,6 +162,10 @@ class LUT3DFrame(BaseFrame):
 										self.lut3d_hdr_minmll_handler)
 		self.lut3d_hdr_maxmll_ctrl.Bind(floatspin.EVT_FLOATSPIN,
 										self.lut3d_hdr_maxmll_handler)
+		self.lut3d_hdr_maxmll_alt_clip_cb.Bind(wx.EVT_CHECKBOX,
+											   self.lut3d_hdr_maxmll_alt_clip_handler)
+		self.lut3d_hdr_sat_ctrl.Bind(wx.EVT_SLIDER,
+									 self.lut3d_hdr_sat_ctrl_handler)
 
 	def lut3d_bind_content_colorspace_handlers(self):
 		self.lut3d_content_colorspace_ctrl.Bind(wx.EVT_CHOICE,
@@ -280,6 +284,11 @@ class LUT3DFrame(BaseFrame):
 			self.panel.Sizer.Layout()
 			self.update_layout()
 		self.panel.Thaw()
+
+	def lut3d_hdr_maxmll_alt_clip_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_maxmll_alt_clip",
+							  int(not self.lut3d_hdr_maxmll_alt_clip_cb.GetValue()))
+		self.lut3d_hdr_update_diffuse_white()
 	
 	def lut3d_apply_cal_ctrl_handler(self, event):
 		setcfg("3dlut.output.profile.apply_cal",
@@ -311,6 +320,11 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_set_option("3dlut.hdr_maxmll",
 							  self.lut3d_hdr_maxmll_ctrl.GetValue())
 
+	def lut3d_hdr_sat_ctrl_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_sat",
+							  self.lut3d_hdr_sat_ctrl.GetValue() / 100.0)
+		self.lut3d_hdr_update_sat_val()
+
 	def lut3d_trc_black_output_offset_ctrl_handler(self, event):
 		if event.GetId() == self.lut3d_trc_black_output_offset_intctrl.GetId():
 			self.lut3d_trc_black_output_offset_ctrl.SetValue(
@@ -596,7 +610,7 @@ class LUT3DFrame(BaseFrame):
 								break
 					else:
 						# Assume OS X
-						volumes = ["/"] + glob.glob("/Volumes/*")
+						volumes = ["/"] + safe_glob("/Volumes/*")
 						for volume in volumes:
 							lut_dir = os.path.join(volume, "Library",
 												   "Application Support",
@@ -849,6 +863,9 @@ class LUT3DFrame(BaseFrame):
 		content_rgb_space[1] = colormath.xyY2XYZ(*content_rgb_space[1])
 		content_rgb_space = colormath.get_rgb_space(content_rgb_space)
 		try:
+			# We need to make sure the link between the original profile object
+			# and the one we're eventually going to change is broken
+			profile_in = ICCP.ICCProfile(profile_in.fileName)
 			self.worker.create_3dlut(profile_in, path, profile_abst,
 									 profile_out, apply_cal=apply_cal,
 									 intent=intent, format=format, size=size,
@@ -862,11 +879,15 @@ class LUT3DFrame(BaseFrame):
 									 apply_black_offset=apply_black_offset,
 									 use_b2a=use_b2a, white_cdm2=white_cdm2,
 									 minmll=minmll, maxmll=maxmll,
+									 use_alternate_master_white_clip=getcfg("3dlut.hdr_maxmll_alt_clip"),
+									 hdr_sat=getcfg("3dlut.hdr_sat"),
 									 ambient_cdm2=ambient_cdm2,
 									 content_rgb_space=content_rgb_space,
 									 hdr_display=getcfg("3dlut.hdr_display"),
 									 XYZwp=XYZwp)
 		except Exception, exception:
+			if exception.__class__.__name__ in dir(exceptions):
+				raise
 			return exception
 		return True
 	
@@ -1335,6 +1356,7 @@ class LUT3DFrame(BaseFrame):
 		setcfg(option, v)
 		if option in ("3dlut.hdr_peak_luminance", "3dlut.hdr_minmll",
 					  "3dlut.hdr_maxmll"):
+			self.lut3d_show_hdr_maxmll_alt_clip_ctrl()
 			self.lut3d_hdr_update_diffuse_white()
 		elif option == "3dlut.hdr_ambient_luminance":
 			self.lut3d_hdr_update_system_gamma()
@@ -1343,7 +1365,8 @@ class LUT3DFrame(BaseFrame):
 		# Update knee start info for BT.2390-3 roll-off
 		bt2390 = colormath.BT2390(0, getcfg("3dlut.hdr_peak_luminance"),
 								  getcfg("3dlut.hdr_minmll"),
-								  getcfg("3dlut.hdr_maxmll"))
+								  getcfg("3dlut.hdr_maxmll"),
+								  getcfg("3dlut.hdr_maxmll_alt_clip"))
 		diffuse_ref_cdm2 = 94.37844
 		diffuse_PQ = colormath.specialpow(diffuse_ref_cdm2 / 10000, 1.0 / -2084)
 		# Determine white cd/m2 after roll-off
@@ -1358,6 +1381,11 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_diffuse_white_txt_label.ForegroundColour = signalcolor
 		self.lut3d_hdr_diffuse_white_txt.ContainingSizer.Layout()
 
+	def lut3d_hdr_update_sat_val(self):
+		v = getcfg("3dlut.hdr_sat") * 100
+		self.lut3d_hdr_sat_ctrl_lum_val.Label = "%i%%" % (100 - v)
+		self.lut3d_hdr_sat_ctrl_sat_val.Label = "%i%%" % v
+
 	def lut3d_hdr_update_system_gamma(self):
 		# Update system gamma for HLG based on ambient luminance (BT.2390-3)
 		hlg = colormath.HLG(ambient_cdm2=getcfg("3dlut.hdr_ambient_luminance"))
@@ -1427,6 +1455,7 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_peak_luminance_ctrl.SetValue(getcfg("3dlut.hdr_peak_luminance"))
 		self.lut3d_hdr_minmll_ctrl.SetValue(getcfg("3dlut.hdr_minmll"))
 		self.lut3d_hdr_maxmll_ctrl.SetValue(getcfg("3dlut.hdr_maxmll"))
+		self.lut3d_hdr_maxmll_alt_clip_cb.SetValue(not bool(getcfg("3dlut.hdr_maxmll_alt_clip")))
 		self.lut3d_hdr_update_diffuse_white()
 		self.lut3d_hdr_ambient_luminance_ctrl.SetValue(getcfg("3dlut.hdr_ambient_luminance"))
 		self.lut3d_hdr_update_system_gamma()
@@ -1445,13 +1474,14 @@ class LUT3DFrame(BaseFrame):
 		else:
 			i = self.lut3d_content_colorspace_ctrl.Count - 1
 		self.lut3d_content_colorspace_ctrl.SetSelection(i)
+		self.lut3d_hdr_sat_ctrl.SetValue(int(round(getcfg("3dlut.hdr_sat") * 100)))
+		self.lut3d_hdr_update_sat_val()
 
 	def update_linking_controls(self):
 		self.gamut_mapping_inverse_a2b.SetValue(
 			not getcfg("3dlut.gamap.use_b2a"))
 		self.gamut_mapping_b2a.SetValue(
 			bool(getcfg("3dlut.gamap.use_b2a")))
-		self.lut3d_show_trc_controls()
 		if (hasattr(self, "input_profile") and
 			"rTRC" in self.input_profile.tags and
 			"gTRC" in self.input_profile.tags and
@@ -1513,6 +1543,7 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_trc_apply_ctrl.SetValue(True)
 		else:
 			self.lut3d_trc_apply_none_ctrl.SetValue(True)
+		self.lut3d_show_trc_controls()
 	
 	def lut3d_show_bitdepth_controls(self):
 		frozen = self.IsFrozen()
@@ -1539,6 +1570,14 @@ class LUT3DFrame(BaseFrame):
 							   getcfg("3dlut.trc").startswith("smpte2084") and
 							   getcfg("3dlut.format") == "madVR")
 
+	def lut3d_show_hdr_maxmll_alt_clip_ctrl(self):
+		self.panel.Freeze()
+		show = self.lut3d_hdr_maxmll_ctrl.IsShown()  # BT.2390 (roll-off)
+		self.lut3d_hdr_maxmll_alt_clip_cb.Show(show and
+											   getcfg("3dlut.hdr_maxmll") < 10000)
+		self.panel.Layout()
+		self.panel.Thaw()
+
 	def lut3d_show_trc_controls(self, show=True):
 		self.panel.Freeze()
 		show = show and self.worker.argyll_version >= [1, 6]
@@ -1568,6 +1607,7 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_maxmll_label.Show(show and smpte2084r)
 		self.lut3d_hdr_maxmll_ctrl.Show(show and smpte2084r)
 		self.lut3d_hdr_maxmll_ctrl_label.Show(show and smpte2084r)
+		self.lut3d_show_hdr_maxmll_alt_clip_ctrl()
 		self.lut3d_hdr_diffuse_white_label.Show(show and smpte2084r)
 		self.lut3d_hdr_diffuse_white_txt.Show(show and smpte2084r)
 		self.lut3d_hdr_diffuse_white_txt_label.Show(show and smpte2084r)
@@ -1577,6 +1617,8 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_system_gamma_label.Show(show and hlg)
 		self.lut3d_hdr_system_gamma_txt.Show(show and hlg)
 		show = (show or smpte2084) and not hlg
+		sizer = self.lut3d_hdr_sat_ctrl.ContainingSizer
+		sizer.ShowItems(show and smpte2084)
 		show = show and ((hasattr(self, "lut3d_create_cb") and
 						  getcfg("3dlut.create")) or self.XYZbpout > [0, 0, 0])
 		self.lut3d_trc_gamma_type_ctrl.Show(show and not hdr)
